import random
import math
random.seed("basic01/cooking")

class TestGen:
    def __init__(self):
        self.test_number = 0
        self.alphabet = 'abcdefghijklmnopqrstuvwxyz_'

    def generate_samples(self):
        self.print_test(['onion', 'pepper', 'tomato_paste', 'mayonnaise'],
                        [['onion', 11], ['pepper_black', 3], ['pepper_red', 5], ['mayonnaise', 30], ['tomato_paste', 40], ['tomato', 20]],
                        [['pepper', 'pepper_red'], ['tomato_paste', 'tomato']])
        self.print_test(['a', 'b', 'c'],
                        [['a', 10], ['b', 10], ['c', 10], ['e', 5], ['f', 4]],
                        [['a', 'b', 'd'], ['c', 'e', 'f'], ['b', 'c', 'f']])
        self.print_test(['a', 'b', 'c'],
                        [['b', 10], ['c', 10], ['e', 5], ['f', 4]],
                        [['a', 'b', 'd'], ['c', 'e', 'f'], ['b', 'c', 'f']])

    def generate_random_name(self, length):
        s = ''
        for i in range(length):
            s += self.alphabet[random.randint(0, len(self.alphabet) - 1)]
        return s

    def generate_min_tests(self):
        self.print_test(['a', 'b', 'c'], [['a', 1], ['b', 2]], [])
        self.print_test(['a', 'b', 'c'], [['a', 1], ['b', 2], ['c', 3]], [])
        self.print_test(['a'], [['a', 1], ['b', 2]], [['a', 'b']])
        self.print_test(['a'], [['a', 2], ['b', 2]], [['a', 'b']])
        self.print_test(['a'], [['a', 3], ['b', 2]], [['a', 'b']])
        self.print_test(['a'], [['a', 3], ['b', 2]], [['a', 'b', 'c']])
        self.print_test(['a'], [['a', 3], ['b', 2], ['c', 2]], [['a', 'b', 'c']])
        self.print_test(['a'], [['a', 100], ['b', 2], ['c', 2], ['d', 3], ['e', 4], ['f', 5], ['g', 6]], [['a', 'b', 'c', 'd', 'e', 'f', 'g']])
        self.print_test(['a'], [['a', 100], ['b', 2], ['c', 2], ['d', 3], ['f', 5], ['g', 6]], [['a', 'b', 'c', 'd', 'e', 'f', 'g']])
        self.print_test(['a'], [['a', 100], ['b', 2], ['c', 2], ['d', 3], ['f', 5], ['g', 6]], [['a', 'b', 'c', 'd', 'e', 'f', 'g'], ['x', 'y', 'z'], ['z', 't', 's', 'q']])

    def generate_hand_tests(self):
        pass

    def graph_repr(self, graph, names):
        n = len(graph)
        ans = []
        for i in range(n):
            if len(graph[i]) == 0:
                continue
            e = []
            e.append(names[i])
            e += list(map(lambda x: names[x], graph[i]))
            ans.append(e)
        return ans

    def generate_random_test(self, n, maxlen, maxcost):
        p = [-1 for _ in range(n)]
        graph = [[] for _ in range(n)]
        names = [self.generate_random_name(maxlen) for _ in range(n)]
        for i in range(1, n):
            p[i] = random.randint(0, i - 1)
            graph[p[i]].append(i)
        costs = []
        for i in range(n):
            if len(graph[i]) == 0:
                costs.append([names[i], random.randint(1, maxcost)])
            else:
                if random.randint(1, 3) == 1:
                    costs.append([names[i], random.randint(1, maxcost)])
        v = random.randint(0, n - 1)
        while len(graph[v]) == 0:
            v = random.randint(0, n - 1)
        ps = [i for i in range(n)]
        random.shuffle(ps)
        vs = list(map(lambda x: names[x], ps[:random.randint(1, n - 1)]))
        self.print_test(vs, costs, self.graph_repr(graph, names))

    def generate_random_test_not_all_leaves(self, n, maxlen, maxcost):
        p = [-1 for _ in range(n)]
        graph = [[] for _ in range(n)]
        names = [self.generate_random_name(maxlen) for _ in range(n)]
        for i in range(1, n):
            p[i] = random.randint(0, i - 1)
            graph[p[i]].append(i)
        costs = []
        lv = None
        for i in range(n):
            if len(graph[i]) == 0 and random.randint(1, 4) > 1:
                lv = names[i]
                costs.append([names[i], random.randint(1, maxcost)])
            else:
                if random.randint(1, 3) == 1:
                    costs.append([names[i], random.randint(1, maxcost)])
        if len(costs) == 0:
            costs.append([lv, random.randint(1, maxcost)])
        v = random.randint(0, n - 1)
        while len(graph[v]) == 0:
            v = random.randint(0, n - 1)
        ps = [i for i in range(n)]
        random.shuffle(ps)
        vs = list(map(lambda x: names[x], ps[:random.randint(1, n - 1)]))
        self.print_test(vs, costs, self.graph_repr(graph, names))

    def generate_max_tests(self, maxn, maxlen, maxcost):
        n = maxn
        names = [self.generate_random_name(maxlen) for _ in range(n)]
        costs = []
        for i in range(n):
            costs.append([names[i], maxcost - i])
        graph = []
        for i in range(n - 1):
            graph.append([names[i], names[i + 1]])
        self.print_test(names, costs, graph)

        n = maxn
        names = [self.generate_random_name(maxlen) for _ in range(n)]
        costs = []
        for i in range(n):
            costs.append([names[i], maxcost - (n - i - 1)])
        graph = []
        for i in range(n - 1):
            graph.append([names[i], names[i + 1]])
        self.print_test(names, costs, graph)        

        n = maxn
        names = [self.generate_random_name(maxlen) for _ in range(n)]
        costs = []
        for i in range(n):
            costs.append([names[i], maxcost])
        graph = []
        for i in range(n - 1):
            graph.append([names[i], names[i + 1]])
        self.print_test(names, costs, graph)

        n = maxn
        names = [self.generate_random_name(maxlen) for _ in range(n)]
        costs = []
        for i in range(n):
            costs.append([names[i], random.randint(maxcost - n, maxcost)])
        graph = []
        for i in range(n - 1):
            graph.append([names[i], names[i + 1]])
        self.print_test(names, costs, graph)        


    def print_test(self, a, b, c):
        self.test_number += 1
        print('Generating test #%d' % self.test_number)
        test_name = "{0:0=2d}".format(self.test_number)
        test_file = open(test_name, 'w')
        print(len(a), file=test_file)
        print(' '.join(map(str, a)), file=test_file)
        print(len(b), file=test_file)
        print('\n'.join(map(lambda e: e[0] + ' ' + str(e[1]), b)), file=test_file)
        print(len(c), file=test_file)
        if len(c) == 0:
            print()
        else:
            print('\n'.join(map(lambda e: str(len(list(e)) - 1) + ' ' + ' '.join(e), c)), file=test_file)
        test_file.close()

    def generate_all_tests(self):
        self.generate_samples()
        self.generate_min_tests()
        self.generate_hand_tests()

        for n in [3, 5, 10, 25, 50, 100]:
            for maxcost in [10, 1000, 10**5, 10**9]:
                self.generate_random_test(n, 20, maxcost)
                self.generate_random_test_not_all_leaves(n, 20, maxcost)
                
        self.generate_max_tests(100, 20, 10**9)

import time
start = time.time()
writer = TestGen()
writer.generate_all_tests()
finish = time.time()
print('All tests were generated. Elapsed time: ', finish - start)
