#include "testlib.h"

const int MAX_N = 100;
const int MAX_COST = 1e9;
const int MAX_LEN = 20;

bool check_ingredient(const std::string &s, const char* where, int index) {
  if (1 > s.size() || s.size() > MAX_LEN) {
    quitf(_fail, ("Ingredient #" + std::to_string(index) + " from " + std::string(where) + " has length not in [1, " + std::to_string(MAX_LEN) + "]").c_str());
  }
  for (size_t i = 0; i < s.size(); i++) {
    if (('a' > s[i] || s[i] > 'z') && s[i] != '_') {
      quitf(_fail, ("Ingredient #" + std::to_string(index) + " contains symbols except small Latin and underscore").c_str());
    }
  }
}

bool dfs(std::map<std::string, std::vector<std::string>> &graph, std::map<std::string, int> &was, std::string &v) {
  was[v] = 1;
  bool answer = true;
  for (auto &u : graph[v]) {
    if (was[u] == 1) {
      return false;
    }
    if (was[u] == 0) {
      answer &= dfs(graph, was, u);
    }
  }
  was[v] = 2;
  return answer;
}

int main(int argc, char* argv[])
{
  registerValidation(argc, argv);
  int n = inf.readInt(1, MAX_N, "n");
  inf.readEoln();
  std::set<std::string> ingredients;
  for (int i = 0; i < n; i++) {
    std::string s = inf.readToken();
    ingredients.insert(s);
    check_ingredient(s, "target", i + 1);
    if (i + 1 < n) {
      inf.readSpace();
    } else {
      inf.readEoln();
    }
  }
  int m = inf.readInt(1, MAX_N, "m");
  inf.readEoln();
  std::set<std::string> used;
  for (int i = 0; i < m; i++) {
    std::string s = inf.readToken();
    check_ingredient(s, "market", i + 1);
    if (used.count(s) > 0) {
      quitf(_fail, ("Ingredient #" + std::to_string(i + 1) + " from market is presented twice").c_str());
    }
    used.insert(s);
    inf.readSpace();
    inf.readInt(1, MAX_COST, ("cost[" + std::to_string(i + 1) + "]").c_str());
    inf.readEoln();
  }

  int k = inf.readInt(0, MAX_N - 1, "k");
  inf.readEoln();
  used.clear();
  std::map<std::string, std::vector<std::string>> graph;
  for (int i = 0; i < k; i++) {
    int c = inf.readInt(1, MAX_N - 1, ("c[" + std::to_string(i + 1) + "]"));
    inf.readSpace();
    std::string s = inf.readToken();
    check_ingredient(s, "receipt", i + 1);
    if (used.count(s) > 0) {
      quitf(_fail, ("Target ingredient from receipt #" + std::to_string(i + 1) + " is specified twice").c_str()); 
    }
    used.insert(s);
    inf.readSpace();
    std::set<std::string> receipt;
    for (int j = 0; j < c; j++) {
      std::string si = inf.readToken();
      check_ingredient(si, ("receipt #" + std::to_string(i + 1)).c_str(), j + 1);
      graph[s].push_back(si);
      receipt.insert(si);
      if (j + 1 < c) {
        inf.readSpace();
      } else {
        inf.readEoln();
      }
    }
    if (receipt.size() != c) {
      quitf(_fail, ("Ingredients in receipt #" + std::to_string(i + 1) + " are not pairwise distinct").c_str());      
    }
  }

  std::map<std::string, int> was;
  for (const auto &v : graph) {
    was.clear();
    std::string x = v.first;
    if (!dfs(graph, was, x)) {
      quitf(_fail, "Graph is not acyclic");
    }
  }
  inf.readEof();
  return 0;
}
