#include "testlib.h"
#include <iostream>
#include <stdio.h>
#include <cmath>
#include <vector>
#include <algorithm>
#include <set>
#include <map>
#include <queue>
#include <deque>
#include <stack>
#include <string>
#include <bitset>
#include <ctime>
#pragma comment(linker, "/STACK:256000000")
using namespace std;

#define in(s) freopen(s, "r", stdin)
#define out(s) freopen(s, "w", stdout)
#define forn(i, n) for (int i = 0; i < n; i++)
#define endl '\n'
#define mp make_pair

typedef long long ll;
typedef long double ld;

struct vec
{
	ll x, y;

	vec()
	{
		this->x = 0ll;
		this->y = 0ll;
	}

	vec(ll nx, ll ny)
	{
		this->x = nx;
		this->y = ny;
	}

	vec operator+(const vec &b) const
	{
		return vec(x + b.x, y + b.y);
	}
	vec operator-(const vec &b) const
	{
		return vec(x - b.x, y - b.y);
	}
	ll operator%(const vec &b) const
	{
		return x * b.y - y * b.x;
	}
	ll operator*(const vec &b) const
	{
		return x * b.x + y * b.y;
	}
	ll len()
	{
		return x * x + y * y;
	}
};

bool less1(vec a, vec b)
{
	return a % b > 0ll;
}

vector <vec> edge, convexhull_without_part_points;

bool check_trian(vec a, vec b, vec c, vec q)
{
	ll S = abs((a - b) % (c - b));
	ll s1 = abs((a - q) % (b - q));
	ll s2 = abs((b - q) % (c - q));
	ll s3 = abs((c - q) % (a - q));
	return (S == s1 + s2 + s3);
}

bool inside(vec q)
{
	if (q.x < convexhull_without_part_points[0].x)
		return false;
	q = q - convexhull_without_part_points[0];
	if (q % edge[0] > 0ll || edge.back() % q > 0ll)
		return false;

	if ((int)edge.size() == 1)
	{
		return ((q % edge[0] == 0ll) && (q.x <= edge[0].x && q.y <= edge[0].y));
	}

	int p = lower_bound(edge.begin(), edge.end(), q, less1) - edge.begin();
	p += (p == 0);
	return check_trian(vec(), edge[p - 1], edge[p], q);
}

bool segments_intersect(vec a, vec c, vec b, vec d)
{
	vec p1 = c - a, p2 = d - b;
	if (p1 % p2 == 0ll)
	{
		if ((c - a) % (c - b) != 0ll)
		{
			return false;
		}
		bool flag1 = (a - b) * (c - b) > 0ll;
		bool flag2 = (a - d) * (c - d) > 0ll;
		bool flag3 = (c - b) * (c - d) > 0ll;
		bool flag4 = (a - b) * (a - d) > 0ll;
		if (flag1 && flag2 && flag3 && flag4)
			return false;
		else
			return true;
	}
	else
	{
		bool flag1 = ((b - a) % (d - a) > 0ll) && ((b - c) % (d - c) > 0ll);
		bool flag2 = ((b - a) % (d - a) < 0ll) && ((b - c) % (d - c) < 0ll);
		bool flag3 = ((c - b) % (a - b) > 0ll) && ((c - d) % (a - d) > 0ll);
		bool flag4 = ((c - b) % (a - b) < 0ll) && ((c - d) % (a - d) < 0ll);
		if (!flag1 && !flag2 && !flag3 && !flag4)
			return true;
		else
			return false;
	}
}

bool cmp1(pair<vec, int> a, pair<vec, int> b)
{
	if (a.first.x != b.first.x)
		return a.first.x < b.first.x;
	else
	{
		if (a.first.y != b.first.y)
			return a.first.y < b.first.y;
		else
			return a.second < b.second;
	}
}

bool cmp2(pair<vec, int> a, pair<vec, int> b)
{
	if (a.first % b.first == 0ll)
		return a.first.len() < b.first.len();
	else
		return a.first % b.first > 0ll;
}

void solve(vector <vec> have, vector <vec> other1)
{
	int n = (int)have.size();
	vector < pair<vec, int> > arr(n);
	forn(i, n)
		arr[i] = mp(have[i], i);

	int m = (int)other1.size();
	vector < pair<vec, int> > other(m);
	for (int i = 0; i < m; i++)
		other[i] = mp(other1[i], i);

	sort(arr.begin(), arr.end(), cmp1);
	vec start = arr[0].first;

	forn(i, n)
		arr[i].first = arr[i].first - start;

	sort(arr.begin(), arr.end(), cmp2);

	vector < pair<vec, int> > q;
	forn(i, n)
	{
		if ((int)q.size() < 2)
		{
			q.push_back(arr[i]);
			continue;
		}
		while ((int)q.size() > 1)
		{
			vec a = q[(int)q.size() - 2].first - q.back().first;
			vec b = arr[i].first - q.back().first;

			if (a % b < 0ll)
				break;
			q.pop_back();
		}
		q.push_back(arr[i]);
	}

	for (int i = 0; i < (int)q.size(); i++)
	{
		q[i].first = q[i].first + start;
	}

	for (int i = 0; i < (int)q.size(); i++)
	{
		convexhull_without_part_points.push_back(q[i].first);
	}
	for (int i = 1; i < (int)q.size(); i++)
	{
		edge.push_back(q[i].first - q[0].first);
	}

	if (m == 1)
	{
		if (inside(other[0].first))
		{
			quitf(_ok, "answer is ok");
		}
		else
		{
			quitf(_wa, "Convex hulls don't intersect");
		}
	}

	if (inside(other[0].first) || inside(other[1].first))
	{
		quitf(_ok, "answer is ok");
	}

	bool flag = false;
	for (int i = 0; i < (int)q.size(); i++)
	{
		flag |= segments_intersect(other[0].first, other[1].first, q[i].first, q[(i + 1) % (int)q.size()].first);
	}

	if (flag)
	{
		quitf(_ok, "answer is ok");
	}
	else
	{
		quitf(_wa, "Convex hulls don't intersect");
	}

	return;
}


int main(int argc, char**argv)
{
	setName("Check partipiciant's answer");
	registerTestlibCmd(argc, argv);

	// ans - jury's answer
	// ouf - partipiciant's answer
	// inf - test

	int ans_n = ans.readInt();
	int part_n = ouf.readInt();

	if (ans_n != part_n)
	{
		quitf(_wa, "Incorrect optimal set's size. Expected %d, found %d", ans_n, part_n);
	}

	int n = inf.readInt();

	vector <vec> arr(n);
	multiset < pair<ll, ll> > mss;
	for (int i = 0; i < n; i++)
	{
		arr[i].x = inf.readInt();
		arr[i].y = inf.readInt();
		mss.insert(mp(arr[i].x, arr[i].y));
	}

	vector <int> part_points;

	while (!ouf.seekEof())
	{
		int cur_point = ouf.readInt();
		part_points.push_back(cur_point);
	}

	if ((int)part_points.size() != part_n)
	{
		quitf(_wa, "Count of given indexes is not equal to given set's size");
	}

	set <int> ss;
	for (int i = 0; i < (int)part_points.size(); i++)
	{
		ss.insert(part_points[i]);
	}

	if ((int)ss.size() != (int)part_points.size())
	{
		quitf(_wa, "Set of given indexes has equal elements");
	}

	for (int i = 0; i < (int)part_points.size(); i++)
	{
		if (part_points[i] < 1 || part_points[i] > n)
		{
			quitf(_wa, "Given indexes has incorrect numbers.");
		}
	}

	vector <bool> close(n);
	for (int i = 0; i < (int)part_points.size(); i++)
	{
		close[part_points[i] - 1] = true;

		pair <ll, ll> cur_part_pair = mp(arr[part_points[i] - 1].x, arr[part_points[i] - 1].y);
		mss.erase(mss.find(cur_part_pair));
		if (mss.find(cur_part_pair) != mss.end())
		{
			quitf(_ok, "answer is ok");
		}
	}

	vector <vec> arr_without_part_points;
	vector <vec> part_arr;
	for (int i = 0; i < n; i++)
	{
		if (!close[i])
			arr_without_part_points.push_back(arr[i]);
		else
			part_arr.push_back(arr[i]);
	}

	solve(arr_without_part_points, part_arr);

	quitf(_ok, "answer is ok");

	return 0;
}